local name, addon = ...

local playerID,guildID,db,options
local NUM_BAG_SLOTS = NUM_BAG_SLOTS
local GetContainerNumSlots = GetContainerNumSlots
local GetContainerItemLink = GetContainerItemLink
local GetContainerItemInfo = GetContainerItemInfo
local strsplit,format = strsplit,format
local resultStore = {}
local sync = {}
local moduls = {}
local defaults = {
	['bagsToggle'] = true,
	['bankToggle'] = true,
	['charToggle'] = true,
	['showID'] = true,
	['icons'] = {},
	['names'] = {},
	['currencies'] = {},
	['bags'] = {},
	['bank'] = {},
	['guildbank'] = {},
	['units'] = {},
	['guilds'] = {}
}
local function getPlayerID()
	if(playerID) then
		return playerID
	end
	local pname, pserver = UnitFullName("player")
	if(not pserver) then
		return
	end
	local player = format("%s - %s",pname,pserver)
	playerID = 0
	db['units'] = db['units'] or {}
	for k,v in pairs(db['units']) do
		if(v['name']==player) then
			playerID = k
			return playerID
		end
		if(k>=playerID) then
			playerID = k
		end
	end
	playerID = playerID + 1
	db['units'][playerID]= {
		['name'] = player,
		['faction'] = UnitFactionGroup('player')
	}
	return playerID
end
local function checkPID()
	if(not playerID) then
		playerID = getPlayerID()
	end
	if(playerID) then
		db['bags'][playerID] = db['bags'][playerID] or {}
		db['bank'][playerID] = db['bank'][playerID] or {}
		db['currencies'][playerID] = db['currencies'][playerID] or {}
		return true
	end
end
addon['getPID'] = getPlayerID
addon['checkPID'] = checkPID
local function getGuildID()
	local pname, pserver = UnitFullName("player")
	if(not pserver) then
		return
	end
	local guildName, guildRankName, guildRankIndex, guildRealm = GetGuildInfo("player")
	guildID = 0
	if(not guildName) then return end
	local guild = format("%s - %s",guildName,guildRealm and guildRealm or pserver)
	db['guilds'] = db['guilds'] or {}
	for k,v in pairs(db['guilds']) do
		if(v['name']==guild) then
			guildID = k
			return guildID
		end
		if(k>=guildID) then
			guildID = k
		end
	end
	guildID = guildID + 1
	db['guilds'][playerID]= {
		['name'] = guild,
		['faction'] = UnitFactionGroup('player')
	}
	return playerID	
end
addon['getGID'] = getGuildID
local function checkGID()
	if(not guildID) then
		guildID = getGuildID()
	end
	if(guildID and guildID>0) then
		db['guildbank'][guildID] = db['guildbank'][guildID] or {}
		return true
	end
end
addon['checkGID'] = checkGID
function addon.getGuilds()
	return db['guilds']
end
local function setName(id,itemname)
	if(id and itemname) then
		db['names'][id] = itemname
	end
end
local function getName(id)
	if(not db['names'][id]) then
		setName(id,GetItemInfo(id))
	end
	return db['names'][id]
end
addon['getName'] = getName
addon['setIcon'] = function(id,icon)
	if(id and icon) then
		db['icons'][id] = "|T"..icon..":0|t"
	end
end
local function getIcon(id)
	if(not db['icons'][id]) then
		local itemname, _, _, _, _, _, _, _, _, icon = GetItemInfo(id)
		setName(id,itemname)
		addon['setIcon'](id,icon)
	end
	return db['icons'][id] or ''
end
addon['getIcon'] = getIcon
addon['getUnits'] = function()
	return db['units']
end
local function getResult(id)
	if(resultStore[id]) then
		return resultStore[id]
	end
end
addon['getResult'] = getResult
local function setResult(id,res)
	resultStore[id] = res
end
addon['setResult'] = setResult

local function search(id,res)
	res = res or {}
	for k,modul in pairs(moduls) do
		if(type(modul.Search)=='function') then
			res = modul.Search(id,res)
		end
	end
	return res
end
local function output(input,sum,b)
	input = input or {}
	sum = sum or 0
	for k,modul in pairs(moduls) do
		if(type(modul.Output)=='function') then
			input,sum = modul.Output(input,sum,b)
		end
	end
	return input,sum
end
local function specialoutput(tooltip, id)
	for k,modul in pairs(moduls) do
		if(type(modul.Search2)=='function') then
			local res = modul.Search2(id)
			if(res) then
				for k,v in pairs(res) do
					tooltip:AddLine(v)
				end
			end
		end
	end
end
local function GameTooltip_OnTooltipSetItem(tooltip)
	local _, link = tooltip:GetItem()
	if(not link) then
		return
	end
	local id = addon['getItemId'](link)
	if(id)then
		tooltip:AddLine()
		local res = getResult(id)
		if(not res) then
			res = search(id)
			setResult(id,res)
		end
		for name,b in pairs(res) do
			local dname, drealm = strsplit(" - ",name)
			local input,sum = output(input,sum,b)
			tooltip:AddLine(format("%s %s: %s [ %s ] ",getIcon(id),dname,sum,addon['colorize'](table.concat(input,', '),"ffffff")))
			input = nil
		end
		res = specialoutput(tooltip, id)
		if(db['showID']) then
			tooltip:AddLine(format("Id: %i",id))
		end
	end
end
GameTooltip:HookScript("OnTooltipSetItem", GameTooltip_OnTooltipSetItem)
addon['funcSync'] = function(func,delayed)
	local st = GetTime()
	if(delayed and sync[func]['times'][delayed]) then
		sync[func]['times'][delayed] = nil
	end
	sync[func] = sync[func] or {
		['last'] = 0,
		['times'] = {}
	}
	if(not (st == sync[func]['last'])) then
		sync[func]['last'] = st
		return true
	else
		if(not sync[func]['times'][st]) then
			sync[func]['times'][st] = true
			C_Timer.After(.1, function() func(st) end)
		end
		return
	end	
end
addon['GET_ITEM_INFO_RECEIVED'] = function(self, event, id)
	if(not db['names'][id]) then
		setName(id,GetItemInfo(id))
	end
end
addon['wipeResults'] = function()
	wipe(resultStore)
end
local function specialoutput2(id,res)
	for k,modul in pairs(moduls) do
		if(type(modul.Search2)=='function') then
			res = modul.Search2(id,res,true)
		end
	end
	return res
end
SlashCmdList[string.upper(name)] = function(msg)
	local um = string.upper(msg)
	for id, itemname in pairs(db['names']) do
		local un = string.upper(itemname)
		if(string.find(un,um)) then
			local _,itemlink = GetItemInfo(id)
			local dout = {}
			if(id)then
				local res = getResult(id)
				if(not res) then
					res = search(id)
					setResult(id,res)
				end
				for name,b in pairs(res) do
					local dname, drealm = strsplit(" - ",name)
					local input,sum = output(input,sum,b)
					dout[#dout+1] = (format(" %s: %s [ %s ] ",dname,sum,addon['colorize'](table.concat(input,', '),"ffffff")))
					input = nil
				end
				dout = specialoutput2(id,dout)
			end
			if(#dout>0) then
				addon['Print'](format("%s %s",getIcon(id),itemlink and itemlink or itemname))
				for k,v in pairs(dout) do
					addon['Print'](v)
				end
			end
		end
	end
end
_G["SLASH_"..string.upper(name).."1"] = "/find"
addon['GetNamespace'] = function(namespace)
	if(namespace) then
		db[namespace] = db[namespace] or {}
		return db[namespace]
	end
end
addon['RegisterModul'] = function(modul, namespace)
	moduls[#moduls+1] = modul
	addon:AddConfigEntry(name,"toggle",namespace.."Toggle",namespace,"Enable/Disable "..namespace.." on next reload",1,nil,nil,nil,nil,addon['options']['args'][name])
	return addon['GetNamespace'](namespace), db[namespace.."Toggle"]
end

local function OnEvent(self, event, ...)
	if(type(addon[event])=='function') then
		addon[event](self,event,...)
	end
end
local function getPref(pref)
	return db[pref[#pref]]
end

local function setPref(pref,value)
	db[pref[#pref]] = value
	for k, modul in pairs(moduls) do
		if(modul.name .. 'Toggle' == pref[#pref]) then
			if(value) then
				if(type(modul.Enable)=='function') then
					modul.Enable()
				end
			else
				if(type(modul.Disable)=='function') then
					modul.Disable()
				end			
			end
		elseif(type(modul.setPref)=='function' and string.sub(pref[#pref],0,string.len(modul.name)) == modul.name) then
			modul.setPref(pref[#pref], value)
		end
	end
end
local function init(self, ...)
	options = addon:InitConfig(name, true, {
		['type'] = "launcher",
		['icon'] ="Interface\\Icons\\INV_Misc_Bag_10_Blue",
	}, getPref, setPref)
	db = addon['db']['profile'][name]
	addon['RegisterFunc']({
		"GET_ITEM_INFO_RECEIVED"
	},"OnEvent", OnEvent)
	addon:AddConfigEntry(name,"toggle","showID","Show ItemID","Show ItemID in tooltips",1,nil,nil,nil,nil,addon['options']['args'][name])
end
addon['startup'](addon, name, name, init, true, defaults)